# Copyright (c) 2013 Yubico AB
# All rights reserved.
#
#   Redistribution and use in source and binary forms, with or
#   without modification, are permitted provided that the following
#   conditions are met:
#
#    1. Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from __future__ import absolute_import


__dependencies__ = []
__all__ = ['get_version', 'setup', 'release']


from setuptools import setup as _setup, find_packages, Command
from setuptools.command.sdist import sdist
from distutils import log
from distutils.errors import DistutilsSetupError
from datetime import date
from glob import glob
import os
import re

VERSION_PATTERN = re.compile(r"(?m)^__version__\s*=\s*['\"](.+)['\"]$")
DEPENDENCY_PATTERN = re.compile(
    r"(?m)__dependencies__\s*=\s*\[((['\"].+['\"]\s*(,\s*)?)+)\]")

base_module = __name__.rsplit('.', 1)[0]


def get_version(module_name_or_file=None):
    """Return the current version as defined by the given module/file."""

    if module_name_or_file is None:
        parts = base_module.split('.')
        module_name_or_file = parts[0] if len(parts) > 1 else \
            find_packages(exclude=['test', 'test.*'])[0]

    if os.path.isdir(module_name_or_file):
        module_name_or_file = os.path.join(module_name_or_file, '__init__.py')

    with open(module_name_or_file, 'r') as f:
        match = VERSION_PATTERN.search(f.read())
        return match.group(1)


def get_dependencies(module):
    basedir = os.path.dirname(__file__)
    fn = os.path.join(basedir, module + '.py')
    if os.path.isfile(fn):
        with open(fn, 'r') as f:
            match = DEPENDENCY_PATTERN.search(f.read())
            if match:
                return [s.strip().strip('"\'')
                        for s in match.group(1).split(',')]
    return []


def get_package(module):
    return base_module + '.' + module


def setup(**kwargs):
    # TODO: Find a better way to pass this to a command.
    os.environ['setup_long_name'] = kwargs.pop('long_name', kwargs.get('name'))

    if 'version' not in kwargs:
        kwargs['version'] = get_version()
    packages = kwargs.setdefault(
        'packages',
        find_packages(exclude=['test', 'test.*', base_module + '.*']))
    packages.append(__name__)
    install_requires = kwargs.setdefault('install_requires', [])
    for yc_module in kwargs.pop('yc_requires', []):
        packages.append(get_package(yc_module))
        for dep in get_dependencies(yc_module):
            if dep not in install_requires:
                install_requires.append(dep)
    cmdclass = kwargs.setdefault('cmdclass', {})
    cmdclass.setdefault('release', release)
    cmdclass.setdefault('build_man', build_man)
    cmdclass.setdefault('sdist', custom_sdist)
    return _setup(**kwargs)


class custom_sdist(sdist):
    def run(self):
        self.run_command('build_man')

        # Run if available:
        if 'qt_resources' in self.distribution.cmdclass:
            self.run_command('qt_resources')

        sdist.run(self)


class build_man(Command):
    description = "create man pages from asciidoc source"
    user_options = []
    boolean_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        self.cwd = os.getcwd()
        self.fullname = self.distribution.get_fullname()
        self.name = self.distribution.get_name()
        self.version = self.distribution.get_version()

    def run(self):
        if os.getcwd() != self.cwd:
            raise DistutilsSetupError("Must be in package root!")

        for fname in glob(os.path.join('man', '*.adoc')):
            self.announce("Converting: " + fname, log.INFO)
            self.execute(os.system,
                         ('a2x -d manpage -f manpage "%s"' % fname,))


class release(Command):
    description = "create and release a new version"
    user_options = [
        ('keyid', None, "GPG key to sign with"),
        ('skip-tests', None, "skip running the tests"),
        ('pypi', None, "publish to pypi"),
    ]
    boolean_options = ['skip-tests', 'pypi']

    def initialize_options(self):
        self.keyid = None
        self.skip_tests = 0
        self.pypi = 0

    def finalize_options(self):
        self.cwd = os.getcwd()
        self.fullname = self.distribution.get_fullname()
        self.name = self.distribution.get_name()
        self.version = self.distribution.get_version()

    def _verify_version(self):
        with open('NEWS', 'r') as news_file:
            line = news_file.readline()
        now = date.today().strftime('%Y-%m-%d')
        if not re.search(r'Version %s \(released %s\)' % (self.version, now),
                         line):
            raise DistutilsSetupError("Incorrect date/version in NEWS!")

    def _verify_tag(self):
        if os.system('git tag | grep -q "^%s\$"' % self.fullname) == 0:
            raise DistutilsSetupError(
                "Tag '%s' already exists!" % self.fullname)

    def _verify_not_dirty(self):
        if os.system('git diff --shortstat | grep -q "."') == 0:
            raise DistutilsSetupError("Git has uncommitted changes!")

    def _sign(self):
        if os.path.isfile('dist/%s.tar.gz.asc' % self.fullname):
            # Signature exists from upload, re-use it:
            sign_opts = ['--output dist/%s.tar.gz.sig' % self.fullname,
                         '--dearmor dist/%s.tar.gz.asc' % self.fullname]
        else:
            # No signature, create it:
            sign_opts = ['--detach-sign', 'dist/%s.tar.gz' % self.fullname]
            if self.keyid:
                sign_opts.insert(1, '--default-key ' + self.keyid)
        self.execute(os.system, ('gpg ' + (' '.join(sign_opts)),))

        if os.system('gpg --verify dist/%s.tar.gz.sig' % self.fullname) != 0:
            raise DistutilsSetupError("Error verifying signature!")

    def _tag(self):
        tag_opts = ['-s', '-m ' + self.fullname, self.fullname]
        if self.keyid:
            tag_opts[0] = '-u ' + self.keyid
        self.execute(os.system, ('git tag ' + (' '.join(tag_opts)),))

    def run(self):
        if os.getcwd() != self.cwd:
            raise DistutilsSetupError("Must be in package root!")

        self._verify_version()
        self._verify_tag()
        self._verify_not_dirty()
        self.run_command('check')

        self.execute(os.system, ('git2cl > ChangeLog',))

        self.run_command('sdist')

        if not self.skip_tests:
            try:
                self.run_command('test')
            except SystemExit as e:
                if e.code != 0:
                    raise DistutilsSetupError("There were test failures!")

        if self.pypi:
            cmd_obj = self.distribution.get_command_obj('upload')
            cmd_obj.sign = True
            if self.keyid:
                cmd_obj.identity = self.keyid
            self.run_command('upload')

        self._sign()
        self._tag()

        self.announce("Release complete! Don't forget to:", log.INFO)
        self.announce("")
        self.announce("    git push && git push --tags", log.INFO)
        self.announce("")



"""
helper functions to work with OATH HOTP (RFC4226) OTP's and YubiHSM
"""

# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import string
import struct

__all__ = [
    # constants
    # functions
    'validate_oath_hotp_with_aead',
    # classes
 ]

import pyhsm.exception
import pyhsm.aead_cmd

def search_for_oath_code(hsm, key_handle, nonce, aead, counter, user_code, look_ahead=1):
    """
    Try to validate an OATH HOTP OTP generated by a token whose secret key is
    available to the YubiHSM through the AEAD.

    The parameter `aead' is either a string, or an instance of YHSM_GeneratedAEAD.

    Returns next counter value on successful auth, and None otherwise.
    """
    key_handle = pyhsm.util.input_validate_key_handle(key_handle)
    nonce = pyhsm.util.input_validate_nonce(nonce, pad = False)
    aead = pyhsm.util.input_validate_aead(aead)
    counter = pyhsm.util.input_validate_int(counter, 'counter')
    user_code = pyhsm.util.input_validate_int(user_code, 'user_code')
    hsm.load_temp_key(nonce, key_handle, aead)
    # User might have produced codes never sent to us, so we support trying look_ahead
    # codes to see if we find the user's current code.
    for j in xrange(look_ahead):
        this_counter = counter + j
        secret = struct.pack("> Q", this_counter)
        hmac_result = hsm.hmac_sha1(pyhsm.defines.YSM_TEMP_KEY_HANDLE, secret).get_hash()
        this_code = truncate(hmac_result)
        if this_code == user_code:
            return this_counter + 1
    return None

def truncate(hmac_result, length=6):
    """ Perform the truncating. """
    assert(len(hmac_result) == 20)
    offset   =  ord(hmac_result[19]) & 0xf
    bin_code = (ord(hmac_result[offset]) & 0x7f) << 24 \
        | (ord(hmac_result[offset+1]) & 0xff) << 16 \
        | (ord(hmac_result[offset+2]) & 0xff) <<  8 \
        | (ord(hmac_result[offset+3]) & 0xff)
    return bin_code % (10 ** length)



"""
module for talking to the YubiHSM over a socket.
"""

# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

__all__ = [
    # constants
    # functions
    # classes
    'YHSM_Stick_Client',
]

import sys
import re
import socket
import json

import pyhsm.util
import pyhsm.exception

CMD_WRITE = 0
CMD_READ = 1
CMD_FLUSH = 2
CMD_DRAIN = 3
CMD_LOCK = 4
CMD_UNLOCK = 5


DEVICE_PATTERN = re.compile(r'yhsm://(?P<host>[^:]+)(:(?P<port>\d+))?/?')
DEFAULT_PORT = 5348


def pack_data(data):
    if isinstance(data, basestring):
        return data.encode('base64')
    return data


def unpack_data(data):
    if isinstance(data, basestring):
        return data.decode('base64')
    elif isinstance(data, dict) and 'error' in data:
        return pyhsm.exception.YHSM_Error(data['error'])
    return data


def read_sock(sf):
    line = sf.readline()
    return unpack_data(json.loads(line))


def write_sock(sf, cmd, *args):
    json.dump([cmd] + map(pack_data, args), sf)
    sf.write("\n")
    sf.flush()


class YHSM_Stick_Client():
    """
    The current YHSM is a USB device using serial communication.

    This class exposes the basic functions read, write and flush (input).
    """
    def __init__(self, device, timeout=1, debug=False):
        """
        Open YHSM device.
        """
        self.debug = debug
        self.device = device
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        match = DEVICE_PATTERN.match(device)
        host = match.group('host')
        port = match.group('port') or DEFAULT_PORT
        self.socket.connect((host, int(port)))
        self.socket_file = self.socket.makefile('wb')

        self.num_read_bytes = 0
        self.num_write_bytes = 0
        if self.debug:
            sys.stderr.write("%s: OPEN %s\n" % (
                self.__class__.__name__,
                self.socket
            ))
        return None

    def acquire(self):
        write_sock(self.socket_file, CMD_LOCK)
        return self.release

    def release(self):
        write_sock(self.socket_file, CMD_UNLOCK)

    def write(self, data, debug_info=None):
        """
        Write data to YHSM device.
        """
        self.num_write_bytes += len(data)
        if self.debug:
            if not debug_info:
                debug_info = str(len(data))
            sys.stderr.write("%s: WRITE %s:\n%s\n" % (
                self.__class__.__name__,
                debug_info,
                pyhsm.util.hexdump(data)
            ))
        write_sock(self.socket_file, CMD_WRITE, data)
        return read_sock(self.socket_file)

    def read(self, num_bytes, debug_info=None):
        """
        Read a number of bytes from YubiHSM device.
        """
        if self.debug:
            if not debug_info:
                debug_info = str(num_bytes)
            sys.stderr.write("%s: READING %s\n" % (
                self.__class__.__name__,
                debug_info
            ))
        write_sock(self.socket_file, CMD_READ, num_bytes)
        res = read_sock(self.socket_file)
        if isinstance(res, Exception):
            raise res

        if self.debug:
            sys.stderr.write("%s: READ %i:\n%s\n" % (
                self.__class__.__name__,
                len(res),
                pyhsm.util.hexdump(res)
            ))
        self.num_read_bytes += len(res)
        return res

    def flush(self):
        """
        Flush input buffers.
        """
        write_sock(self.socket_file, CMD_FLUSH)
        return read_sock(self.socket_file)

    def drain(self):
        """ Drain input. """
        write_sock(self.socket_file, CMD_DRAIN)
        return read_sock(self.socket_file)

    def raw_device(self):
        """ Get the socket address. Only intended for test code/debugging! """
        return self.socket

    def set_debug(self, new):
        """
        Set debug mode (boolean).

        Returns old setting.
        """
        if type(new) is not bool:
            raise pyhsm.exception.YHSM_WrongInputType(
                'new', bool, type(new))
        old = self.debug
        self.debug = new
        return old

    def __repr__(self):
        return '<%s instance at %s: %s - r:%i w:%i>' % (
            self.__class__.__name__,
            hex(id(self)),
            self.device,
            self.num_read_bytes,
            self.num_write_bytes
        )

    def __del__(self):
        """
        Close device when YHSM instance is destroyed.
        """
        if self.debug:
            sys.stderr.write("%s: CLOSE %s\n" % (
                self.__class__.__name__,
                self.device
            ))
        try:
            self.socket_file.close()
            self.socket.close()
        except Exception:
            pass



# -*- mode: python -*-
# -*- encoding: utf-8 -*-

from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import json
import errno
import pkg_resources
from glob import glob


VS_VERSION_INFO = """
VSVersionInfo(
  ffi=FixedFileInfo(
    # filevers and prodvers should be always a tuple with four
    # items: (1, 2, 3, 4)
    # Set not needed items to zero 0.
    filevers=%(ver_tup)r,
    prodvers=%(ver_tup)r,
    # Contains a bitmask that specifies the valid bits 'flags'r
    mask=0x0,
    # Contains a bitmask that specifies the Boolean attributes
    # of the file.
    flags=0x0,
    # The operating system for which this file was designed.
    # 0x4 - NT and there is no need to change it.
    OS=0x4,
    # The general type of file.
    # 0x1 - the file is an application.
    fileType=0x1,
    # The function of the file.
    # 0x0 - the function is not defined for this fileType
    subtype=0x0,
    # Creation date and time stamp.
    date=(0, 0)
  ),
  kids=[
    StringFileInfo(
      [
      StringTable(
        u'040904E4',
        [StringStruct(u'FileDescription', u'%(name)s'),
        StringStruct(u'FileVersion', u'%(ver_str)s'),
        StringStruct(u'InternalName', u'%(internal_name)s'),
        StringStruct(u'LegalCopyright', u'Copyright © 2015 Yubico'),
        StringStruct(u'OriginalFilename', u'%(exe_name)s'),
        StringStruct(u'ProductName', u'%(name)s'),
        StringStruct(u'ProductVersion', u'%(ver_str)s')])
      ]),
    VarFileInfo([VarStruct(u'Translation', [1033, 1252])])
  ]
)"""

data = json.loads(os.environ['pyinstaller_data'])
try:
    data = dict((k, v.encode('ascii') if isinstance(v, unicode) else v)
                for k, v in data.items())
except NameError:
    pass  # Python 3, encode not needed.
dist = pkg_resources.get_distribution(data['name'])

DEBUG = bool(data['debug'])
NAME = data['long_name']

WIN = sys.platform in ['win32', 'cygwin']
OSX = sys.platform in ['darwin']

ver_str = dist.version
if data['package_version'] > 0:
    ver_str += '.%d' % data['package_version']
file_ext = '.exe' if WIN else ''

if WIN:
    icon_ext = 'ico'
elif OSX:
    icon_ext = 'icns'
else:
    icon_ext = 'png'
ICON = os.path.join('resources', '%s.%s' % (data['name'], icon_ext))

if not os.path.isfile(ICON):
    ICON = None

# Generate scripts from entry_points.
merge = []
entry_map = dist.get_entry_map()
console_scripts = entry_map.get('console_scripts', {})
gui_scripts = entry_map.get('gui_scripts', {})

for ep in list(gui_scripts.values()) + list(console_scripts.values()):
    script_path = os.path.join(WORKPATH, ep.name + '-script.py')
    with open(script_path, 'w') as fh:
        fh.write("import %s\n" % ep.module_name)
        fh.write("%s.%s()\n" % (ep.module_name, '.'.join(ep.attrs)))
    merge.append(
        (Analysis([script_path], [dist.location], None, None, None, None),
         ep.name, ep.name + file_ext)
    )


MERGE(*merge)


# Read version information on Windows.
VERSION = None
if WIN:
    VERSION = 'build/file_version_info.txt'

    global int_or_zero  # Needed due to how this script is invoked

    def int_or_zero(v):
        try:
            return int(v)
        except ValueError:
            return 0

    ver_tup = tuple(int_or_zero(v) for v in ver_str.split('.'))
    # Windows needs 4-tuple.
    if len(ver_tup) < 4:
        ver_tup += (0,) * (4-len(ver_tup))
    elif len(ver_tup) > 4:
        ver_tup = ver_tup[:4]

    # Write version info.
    with open(VERSION, 'w') as f:
        f.write(VS_VERSION_INFO % {
            'name': NAME,
            'internal_name': data['name'],
            'ver_tup': ver_tup,
            'ver_str': ver_str,
            'exe_name': data['name'] + file_ext
        })

pyzs = [PYZ(m[0].pure) for m in merge]

exes = []
for (a, a_name, a_name_ext), pyz in zip(merge, pyzs):
    exe = EXE(pyz,
              a.scripts,
              exclude_binaries=True,
              name=a_name_ext,
              debug=DEBUG,
              strip=None,
              upx=True,
              console=DEBUG or a_name in console_scripts,
              append_pkg=not OSX,
              version=VERSION,
              icon=ICON)
    exes.append(exe)

    # Sign the executable
    if WIN:
        os.system("signtool.exe sign /t http://timestamp.verisign.com/scripts/timstamp.dll \"%s\"" %
                (exe.name))

collect = []
for (a, _, a_name), exe in zip(merge, exes):
    collect += [exe, a.binaries, a.zipfiles, a.datas]

# Data files
collect.append([(os.path.basename(fn), fn, 'DATA') for fn in data['data_files']])

# DLLs, dylibs and executables should go here.
collect.append([(fn[4:], fn, 'BINARY') for fn in glob('lib/*')])

coll = COLLECT(*collect, strip=None, upx=True, name=NAME)

# Write package version for app to display
pversion_fn = os.path.join('dist', NAME, 'package_version.txt')
with open(pversion_fn, 'w') as f:
    f.write(str(data['package_version']))

# Create .app for OSX
if OSX:
    app = BUNDLE(coll,
                 name="%s.app" % NAME,
                 version=ver_str,
                 icon=ICON)

    qt_conf = 'dist/%s.app/Contents/Resources/qt.conf' % NAME
    qt_conf_dir = os.path.dirname(qt_conf)
    try:
        os.makedirs(qt_conf_dir)
    except OSError as e:
        if not (e.errno == errno.EEXIST and os.path.isdir(qt_conf_dir)):
            raise
    with open(qt_conf, 'w') as f:
        f.write('[Path]\nPlugins = plugins')

# Create Windows installer
if WIN:
    installer_cfg = 'resources/win-installer.nsi'
    if os.path.isfile(installer_cfg):
        os.system('makensis.exe -D"VERSION=%s" %s' % (ver_str, installer_cfg))
        installer = "dist/%s-%s-win.exe" % (data['name'], ver_str)
        os.system("signtool.exe sign /t http://timestamp.verisign.com/scripts/timstamp.dll \"%s\"" %
                 (installer))
        print("Installer created: %s" % installer)



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import re
import sys
import os
import time
import unittest
import pyhsm
import pyhsm.util

import test_common

from StringIO import StringIO
from test_common import CfgPassphrase, AdminYubiKeys, HsmPassphrase, PrimaryAdminYubiKey


if __name__ == '__main__':
    class ConfigureYubiHSMforTest(test_common.YHSM_TestCase):

        def test_aaa_echo(self):
            """ Test echo before reconfiguration. """
            self.assertTrue(self.hsm.echo('test'))

        def test_configure_YHSM(self):
            """
            Reconfiguring YubiHSM for tests.
            """
            self.ser = self.hsm.get_raw_device()

            # get the YubiHSM to exit to configuration mode.
            #self.assertTrue(self.hsm.monitor_exit())
            self.hsm.monitor_exit()

            # get the first prompt without sending anything
            self.config_do("", add_cr = False)

            self.config_do("sysinfo")

            self.config_do("help")

            # clear memory and configure as HSM - has a few prompts we have to get past
            #
            if not self.hsm.version.have_key_store_decrypt():
                self.config_do ("hsm ffffffff\r%s\r%s\ryes" % (CfgPassphrase, HsmPassphrase))
            else:
                # HSM> < hsm ffffffff
                # Enabled flags ffffffff = ...
                # Enter cfg password (g to generate)
                # Enter admin Yubikey public id (enter when done)
                # Enter master key (g to generate) yes
                # Confirm current config being erased (type yes)
                AdminYubiKeysStr = '\r'.join(AdminYubiKeys)
                AdminYubiKeysStr += '\r'
                self.config_do ("hsm ffffffff\r%s\r%s\r%s\ryes" % (CfgPassphrase, AdminYubiKeysStr, HsmPassphrase))

            self.hsm.drain()
            self.add_keys(xrange(31))
            self.hsm.drain()

            self.config_do("keylist")

            if self.hsm.version.have_key_store_decrypt():
                self.config_do("keycommit")

            # load a YubiKey (the first Admin YubiKey) into the internal database
            escape_char = chr(27)
            self.config_do("dbload\r00001,%s,%s,%s,\r" % (PrimaryAdminYubiKey) + escape_char, add_cr = False)

            self.config_do("dblist")

            # get back into HSM mode
            sys.stderr.write("exit")
            self.ser.write("exit\r")

            self.hsm.drain()

            self.hsm.reset()

        def test_zzz_unlock(self):
            """ Test unlock of keystore after reconfiguration. """
            if self.hsm.version.have_unlock():
                Params = PrimaryAdminYubiKey
                YK = test_common.FakeYubiKey(pyhsm.yubikey.modhex_decode(Params[0]).decode('hex'),
                                            Params[1].decode('hex'), Params[2].decode('hex')
                                            )
                # After reconfigure, we know the counter values for PrimaryAdminYubiKey is zero
                # in the internal db. However, the test suite initialization will unlock the keystore
                # (in test_common.YHSM_TestCase.setUp) so a value of 0/1 should result in a replayed OTP.
                YK.use_ctr = 0
                YK.session_ctr = 1
                # first verify counters 1/0 gives the expected YSM_OTP_REPLAY
                try:
                    self.hsm.unlock(otp = YK.from_key())
                except pyhsm.exception.YHSM_CommandFailed, e:
                    if e.status != pyhsm.defines.YSM_OTP_REPLAY:
                        raise
                # now do real unlock with values 2/1 (there is an extra unlock done somewhere...)
                YK.use_ctr = 2
                self.assertTrue(self.hsm.unlock(password = HsmPassphrase.decode("hex"), otp = YK.from_key()))
            else:
                self.assertTrue(self.hsm.unlock(password = HsmPassphrase.decode("hex")))

        def test_zzz_echo(self):
            """ Test echo after reconfiguration. """
            self.assertTrue(self.hsm.echo('test'))

        def config_do(self, cmd, add_cr = True):
            # Don't have to output command - it is echoed
            #sys.__stderr__.write("> " + cmd + "\n")
            if add_cr:
                self.ser.write(cmd + "\r")
            else:
                self.ser.write(cmd)
            #time.sleep(0.5)
            recv = ''
            fail_count = 0
            sys.stderr.write("< ")
            while True:
                b = self.ser.read(1)
                if not b:
                    fail_count += 1
                    if fail_count == 5:
                        raise Exception("Did not get the next prompt", recv)
                sys.stderr.write(b)

                recv += b
                lines = recv.split('\n')
                if re.match('^(NO_CFG|WSAPI|HSM).*> .*', lines[-1]):
                    break
            return recv

        def add_keys(self, iterator):
            # Set up one key for every available flag
            for num in iterator:
                flags = 1 << num
                key = ("%02x" % (num + 1)) * 32
                self.add_key(flags, num + 1, key)

            # Set up some extra keys with the same key as the flag-keys, but other flags

            # flags YHSM_OTP_BLOB_VALIDATE (0x200) matching key 0x06 (with flags 0x20, YHSM_BLOB_GENERATE)
            flags = 0x200
            key = "06" * 32
            self.add_key(flags, 0x1000, key)

            # Key with full AES ECB capabilities
            # Enabled flags 0000e000 = YHSM_ECB_BLOCK_ENCRYPT,YHSM_ECB_BLOCK_DECRYPT,YHSM_ECB_BLOCK_DECRYPT_CMP
            flags = 0xe000
            key = "1001" * 16
            self.add_key(flags, 0x1001, key)

            # Key allowed to generate AEAD from known data (loaded into buffer), with user specified noncey
            flags = 0x4 | 0x40000000 | 0x20000000
            key = "1002" * 16
            self.add_key(flags, 0x1002, key)

            # Key with everything enabled at once
            flags = 0xffffffff
            key = "2000" * 16
            self.add_key(flags, 0x2000, key)

            # Key with everything enabled at once, and then revoked
            flags = 0xffffffff
            key = "2001" * 16
            self.add_key(flags, 0x2001, key)
            self.config_do("keydis 2001")

            # Key with NIST test vector for HMAC SHA1
            # Enabled flags 00010000 = YSM_HMAC_SHA1_GENERATE
            flags = 0x10000
            key = "303132333435363738393a3b3c3d3e3f40414243".ljust(64, '0')
            self.add_key(flags, 0x3031, key)

            # Key permitting AEAD generate with user specified nonce
            flags = 0x20000002
            key = "20000002" * 8
            self.add_key(flags, 0x20000002, key)

            # Key permitting random AEAD generate with user specified nonce
            flags = 0x20000008
            key = "20000008" * 8
            self.add_key(flags, 0x20000008, key)

        def add_key(self, flags, num, key):
            keyline = "%08x,%s\r" % (num, key)
            self.config_do("flags %04x" % (flags))
            escape_char = chr(27)
            self.config_do("keyload\r" + keyline + escape_char, add_cr = False)


    unittest.main()



# Copyright (c) 2013 Yubico AB
# All rights reserved.
#
#   Redistribution and use in source and binary forms, with or
#   without modification, are permitted provided that the following
#   conditions are met:
#
#    1. Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from __future__ import absolute_import

from setuptools import Command
from distutils.errors import DistutilsSetupError
import os


__dependencies__ = ['PySide']
__all__ = ['qt_resources']


class _qt_resources(Command):
    description = "convert file resources into code"
    user_options = []
    boolean_options = []
    _source = 'qt_resources'
    _target = ''

    def initialize_options(self):
        pass

    def finalize_options(self):
        self.cwd = os.getcwd()
        self.source = os.path.join(self.cwd, self._source)
        self.target = os.path.join(self.cwd, self._target)

    def _create_qrc(self):
        qrc = os.path.join(self.source, 'qt_resources.qrc')
        with open(qrc, 'w') as f:
            f.write('<RCC>\n<qresource>\n')
            for fname in os.listdir(self.source):
                f.write('<file>%s</file>\n' % fname)
            f.write('</qresource>\n</RCC>\n')
        return qrc

    def run(self):
        if os.getcwd() != self.cwd:
            raise DistutilsSetupError("Must be in package root!")

        qrc = self._create_qrc()
        self.execute(os.system,
                     ('pyside-rcc "%s" -o "%s"' % (qrc, self.target),))
        os.unlink(qrc)

        self.announce("QT resources compiled into %s" % self.target)


def qt_resources(target, sourcedir='qt_resources'):
    target = target.replace('.', os.path.sep)
    if os.path.isdir(target):
        target = os.path.join(target, 'qt_resources.py')
    else:
        target += '.py'

    return type('qt_resources', (_qt_resources, object), {
        '_source': sourcedir,
        '_target': target
    })



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import struct
import unittest
import pyhsm
import pyhsm.oath_hotp

import test_common

class TestOath(test_common.YHSM_TestCase):

    def setUp(self):
        test_common.YHSM_TestCase.setUp(self)

        key = "3132333435363738393031323334353637383930".decode('hex')
	# Enabled flags 00010000 = YSM_HMAC_SHA1_GENERATE
        flags = struct.pack("< I", 0x10000)
        self.nonce = 'f1f2f3f4f5f6'.decode('hex')
        # key 0x2000 has all flags set
        self.key_handle = 0x2000
        self.phantom = pyhsm.defines.YSM_TEMP_KEY_HANDLE

        self.hsm.load_secret(key + flags)
        self.aead = self.hsm.generate_aead(self.nonce, self.key_handle)

        self.assertTrue(isinstance(self.aead, pyhsm.aead_cmd.YHSM_GeneratedAEAD))

        # Load the AEAD into the phantom key handle 0xffffffff.
        self.assertTrue(self.hsm.load_temp_key(self.nonce, self.key_handle, self.aead))

    def test_OATH_HOTP_values(self):
        """ Test OATH HOTP known results. """
        test_vectors = [(0, "cc93cf18508d94934c64b65d8ba7667fb7cde4b0", 755224,),
                        (1, "75a48a19d4cbe100644e8ac1397eea747a2d33ab", 287082,),
                        (2, "0bacb7fa082fef30782211938bc1c5e70416ff44", 359152,),
                        (3, "66c28227d03a2d5529262ff016a1e6ef76557ece", 969429,),
                        (4, "a904c900a64b35909874b33e61c5938a8e15ed1c", 338314,),
                        (5, "a37e783d7b7233c083d4f62926c7a25f238d0316", 254676,),
                        (6, "bc9cd28561042c83f219324d3c607256c03272ae", 287922,),
                        (7, "a4fb960c0bc06e1eabb804e5b397cdc4b45596fa", 162583,),
                        (8, "1b3c89f65e6c9e883012052823443f048b4332db", 399871,),
                        (9, "1637409809a679dc698207310c8c7fc07290d9e5", 520489,),
                        (30, "543c61f8f9aeb35f6dbc3a6847c3fe288cc0ee4c", 26920,),
                        ]

        for c, expected, code in test_vectors:
            hmac_result = self.hsm.hmac_sha1(self.phantom, struct.pack("> Q", c)).get_hash()
            self.assertEqual(expected, hmac_result.encode('hex'))
            self.assertEqual(code, pyhsm.oath_hotp.truncate(hmac_result, length=6))

    def test_OATH_HOTP_validation(self):
        """ Test complete OATH HOTP code validation. """

        oath = lambda counter, user_code, look_ahead: \
            pyhsm.oath_hotp.search_for_oath_code(self.hsm, self.key_handle, self.nonce, self.aead, \
                                                     counter, user_code, look_ahead)

        self.assertEqual(1,	oath(0, 755224, 1))
        self.assertEqual(4,	oath(0, 969429, 4))
        self.assertEqual(None,	oath(0, 969429, 3))
        self.assertEqual(10,	oath(9, 520489, 3))
        self.assertEqual(31,    oath(30, 26920, 1))



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import os
import sys
import unittest
import pyhsm

import test_common

from test_yubikey_validate import YubiKeyEmu

class TestInternalDB(test_common.YHSM_TestCase):

    def setUp(self):
        test_common.YHSM_TestCase.setUp(self)
        self.key = "A" * 16
        self.uid = 'f0f1f2f3f4f5'.decode('hex')
        self.public_id = '4d4d4d4d4d4d'.decode('hex')

    def test_store_yubikey(self):
        """ Test storing a YubiKey in the internal database. """
        # Key handle 0x2000 has all flags enabled
        key_handle = 0x2000

        secret = pyhsm.aead_cmd.YHSM_YubiKeySecret(self.key, self.uid)
        self.hsm.load_secret(secret)

        aead = self.hsm.generate_aead(self.public_id, key_handle)

        # Try to store a record. YSM_ID_DUPLICATE is not an error since we don't
        # always zap the configuration before running the test suite.
        try:
            self.assertTrue(self.hsm.db_store_yubikey(self.public_id, key_handle, aead))
        except pyhsm.exception.YHSM_CommandFailed, e:
            self.assertEqual(e.status, pyhsm.defines.YSM_ID_DUPLICATE)

        # Now, try an invalid validation against that record
        try:
            res = self.hsm.db_validate_yubikey_otp(self.public_id, "x" * 16)
            self.fail("Expected YSM_OTP_INVALID, got %s" % (res))
        except pyhsm.exception.YHSM_CommandFailed, e:
            self.assertEqual(e.status, pyhsm.defines.YSM_OTP_INVALID)

    def test_store_yubikey_with_nonce(self):
        """ Test storing a YubiKey generated with non-public-id nonce in the internal database. """
        if not self.hsm.version.have_YSM_DB_YUBIKEY_AEAD_STORE2():
            raise unittest.SkipTest("Test of command introduced in 1.0.4 disabled.")
        # Key handle 0x2000 has all flags enabled
        key_handle = 0x2000
        public_id = '4d4d4d001122'.decode('hex')
        nonce = '010203040506'.decode('hex')
        key = 'T' * 16
        uid = 'F' * 6

        secret = pyhsm.aead_cmd.YHSM_YubiKeySecret(key, uid)
        self.hsm.load_secret(secret)

        aead = self.hsm.generate_aead(nonce, key_handle)

        # Try to store a record. YSM_ID_DUPLICATE is not an error since we don't
        # always zap the configuration before running the test suite.
        try:
            self.assertTrue(self.hsm.db_store_yubikey(public_id, key_handle, aead, nonce = nonce))
        except pyhsm.exception.YHSM_CommandFailed, e:
            self.assertEqual(e.status, pyhsm.defines.YSM_ID_DUPLICATE)

        # Now, try an invalid validation against that record
        try:
            res = self.hsm.db_validate_yubikey_otp(public_id, "x" * 16)
            self.fail("Expected YSM_OTP_INVALID, got %s" % (res))
        except pyhsm.exception.YHSM_CommandFailed, e:
            self.assertEqual(e.status, pyhsm.defines.YSM_OTP_INVALID)

    def test_real_validate(self):
        """ Test real validation of YubiKey OTP against internal database. """
        # Key handle 0x2000 has all flags enabled
        key_handle = 0x2000

        # randomize last byte of public_id to not have to try so hard to
        # find an unused OTP ;)
        this_public_id = self.public_id[:-1] + os.urandom(1)

        secret = pyhsm.aead_cmd.YHSM_YubiKeySecret(self.key, self.uid)
        self.hsm.load_secret(secret)

        aead = self.hsm.generate_aead(this_public_id, key_handle)

        # Try to store a record. YSM_ID_DUPLICATE is not a duplicate since we don't
        # always zap the configuration before running the test suite.
        try:
            self.assertTrue(self.hsm.db_store_yubikey(this_public_id, key_handle, aead))
        except pyhsm.exception.YHSM_CommandFailed, e:
            self.assertEqual(e.status, pyhsm.defines.YSM_ID_DUPLICATE)

        # OK, now we know there is an entry for this_public_id in the database -
        use_ctr = 0	# the 16 bit power-up counter of the YubiKey
        session_ctr = 0
        timestamp = 0xffff # dunno
        while use_ctr < 0xffff:
            YK = YubiKeyEmu(self.uid, use_ctr, timestamp, session_ctr)
            otp = YK.get_otp(self.key)
            try:
                res = self.hsm.db_validate_yubikey_otp(this_public_id, otp)
                self.assertTrue(isinstance(res, pyhsm.validate_cmd.YHSM_ValidationResult))
                self.assertEqual(res.public_id, this_public_id)
                self.assertEqual(res.use_ctr, use_ctr)
                # OK - if we got here we've got a successful response for this OTP
                break
            except pyhsm.exception.YHSM_CommandFailed, e:
                if e.status != pyhsm.defines.YSM_OTP_REPLAY:
                    raise
            # don't bother with the session_ctr - test run 5 would mean we first have to
            # exhaust 4 * 256 session_ctr increases before the YubiHSM would pass our OTP
            use_ctr += 1

        # Now, check the same OTP again and make sure we get a REPLAY response
        YK = YubiKeyEmu(self.uid, use_ctr, timestamp, session_ctr)
        otp = YK.get_otp(self.key)
        try:
            res = self.hsm.db_validate_yubikey_otp(this_public_id, otp)
            self.fail("Expected YSM_OTP_REPLAY, got %s" % (res))
        except pyhsm.exception.YHSM_CommandFailed, e:
            if e.status != pyhsm.defines.YSM_OTP_REPLAY:
                raise

        # increase session_ctr and test using different method
        session_ctr += 1
        YK = YubiKeyEmu(self.uid, use_ctr, timestamp, session_ctr)
        mh_from_key = YK.from_key(this_public_id, self.key)
        pyhsm.yubikey.validate_otp(self.hsm, mh_from_key)



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import sys
import unittest
import pyhsm

import test_common

class TestOtpValidate(test_common.YHSM_TestCase):

    def setUp(self):
        test_common.YHSM_TestCase.setUp(self)

    def test_load_secret_wrong_key(self):
        """ Test load_secret with key that should not be allowed to. """
        key = "A" * 16
        uid = '\x4d\x4d\x4d\x4d\x4d\x4d'
        public_id = 'f0f1f2f3f4f5'.decode('hex')
        # Enabled flags 00000100 = YHSM_AEAD_STORE
        # HSM> < keyload - Load key data now using flags 00000100. Press ESC to quit
        # 00000009 - stored ok
        key_handle = 9	# Enabled flags 00000020 = YHSM_AEAD_GENERATE

        secret = pyhsm.aead_cmd.YHSM_YubiKeySecret(key, uid)
        self.hsm.load_secret(secret)

        try:
            res = self.hsm.generate_aead(public_id, key_handle)
            self.fail("Expected YSM_FUNCTION_DISABLED, got %s" % (res))
        except pyhsm.exception.YHSM_CommandFailed, e:
            self.assertEquals(e.status, pyhsm.defines.YSM_FUNCTION_DISABLED)

    def test_load_secret(self):
        """ Test load_secret. """
        key = "A" * 16
        uid = '\x4d\x01\x4d\x02'
        public_id = 'f1f2f3f4f5f6'.decode('hex')
        if self.hsm.version.have_YSM_BUFFER_LOAD():
            # Enabled flags 60000004 = YSM_BUFFER_AEAD_GENERATE,YSM_USER_NONCE,YSM_BUFFER_LOAD
            # HSM (keys changed)> < keyload - Load key data now using flags 60000004. Press ESC to quit
            # 00001002 - stored ok
            key_handle = 0x1002
        else:
            # Enabled flags 00000004 = YSM_BUFFER_AEAD_GENERATE
            # HSM> < keyload - Load key data now using flags 00000004. Press ESC to quit
            # 00000003 - stored ok
            key_handle = 3

        secret = pyhsm.aead_cmd.YHSM_YubiKeySecret(key, uid)
        self.hsm.load_secret(secret)

        aead = self.hsm.generate_aead(public_id, key_handle)

        self.assertTrue(isinstance(aead, pyhsm.aead_cmd.YHSM_GeneratedAEAD))

        self.assertEqual(aead.nonce, public_id)
        self.assertEqual(aead.key_handle, key_handle)

    def test_yubikey_secrets(self):
        """ Test the class representing the YUBIKEY_SECRETS struct. """
        aes_128_key = 'a' * 16
        first = pyhsm.aead_cmd.YHSM_YubiKeySecret(aes_128_key, 'b')
        self.assertEqual(len(first.pack()), pyhsm.defines.KEY_SIZE + pyhsm.defines.UID_SIZE)



"""
implementations of basic commands to execute on a YubiHSM
"""

# Copyright (c) 2011-2014 Yubico AB
# See the file COPYING for licence statement.

import struct

__all__ = [
    # constants
    # functions
    # classes
    'YHSM_Cmd_Echo',
    'YHSM_Cmd_System_Info',
    'YHSM_Cmd_Random',
    'YHSM_Cmd_Random_Reseed',
    'YHSM_Cmd_Temp_Key_Load',
    'YHSM_Cmd_Nonce_Get',
    'YHSM_Cmd_Key_Storage_Unlock',
    'YHSM_Cmd_Key_Store_Decrypt',
    'YHSM_Cmd_HSM_Unlock',
    'YHSM_NonceResponse',
]

import pyhsm.defines
import pyhsm.exception
import pyhsm.aead_cmd
from pyhsm.cmd import YHSM_Cmd

class YHSM_Cmd_Echo(YHSM_Cmd):
    """
    Send something to the stick, and expect to get it echoed back.
    """
    def __init__(self, stick, payload=''):
        payload = pyhsm.util.input_validate_str(payload, 'payload', max_len = pyhsm.defines.YSM_MAX_PKT_SIZE - 1)
        # typedef struct {
        #   uint8_t numBytes;                   // Number of bytes in data field
        #   uint8_t data[YSM_MAX_PKT_SIZE - 1]; // Data
        # } YSM_ECHO_REQ;
        packed = chr(len(payload)) + payload
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_ECHO, packed)

    def parse_result(self, data):
        # typedef struct {
        # uint8_t numBytes;                   // Number of bytes in data field
        # uint8_t data[YSM_MAX_PKT_SIZE - 1]; // Data
        # } YSM_ECHO_RESP;
        return data[1:]


class YHSM_Cmd_System_Info(YHSM_Cmd):
    """
    Request system information from the YubiHSM.

    @ivar version_major: Major firmware version
    @ivar version_minor: Minor firmware version
    @ivar version_build: Firmware build version
    @ivar protocol_ver: Communication protocol version
    @ivar system_uid: Unique identifier for YubiHSM
    @type system_uid: string
    """

    version_major = 0
    version_minor = 0
    version_build = 0
    protocol_ver = 0
    system_uid = None

    def __init__(self, stick):
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_SYSTEM_INFO_QUERY)

    def __repr__(self):
        if self.executed:
            return '<%s instance at %s: ver=%s, proto=%s, sysid=0x%s>' % (
                self.__class__.__name__,
                hex(id(self)),
                (self.version_major, self.version_minor, self.version_build),
                self.protocol_ver,
                self.system_uid.encode('hex')
                )
        else:
            return '<%s instance at %s (not executed)>' % (
                self.__class__.__name__,
                hex(id(self))
                )

    def parse_result(self, data):
        # #define SYSTEM_ID_SIZE          12
        # typedef struct {
        #   uint8_t version_major;               // Major version #
        #   uint8_t version_minor;               // Minor version #
        #   uint8_t version_build;               // Build version #
        #   uint8_t protocolVersion;            // Protocol version #
        #   uint8_t systemUid[SYSTEM_ID_SIZE];  // System unique identifier
        # } YHSM_SYSTEM_INFO_RESP;
        self.version_major, \
            self.version_minor, \
            self.version_build, \
            self.protocol_ver, \
            self.system_uid = struct.unpack('BBBB12s', data)
        return self


class YHSM_Cmd_Random(YHSM_Cmd):
    """
    Ask stick to generate a number of random bytes.
    """
    def __init__(self, stick, num_bytes):
        self.num_bytes = pyhsm.util.input_validate_int(num_bytes, 'num_bytes', pyhsm.defines.YSM_MAX_PKT_SIZE - 1)
        # typedef struct {
        #   uint8_t numBytes;                   // Number of bytes to generate
        # } YSM_RANDOM_GENERATE_REQ;
        packed = chr(self.num_bytes)
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_RANDOM_GENERATE, packed)

    def parse_result(self, data):
        # typedef struct {
        #   uint8_t numBytes;                   // Number of bytes generated
        #   uint8_t rnd[YSM_MAX_PKT_SIZE - 1];  // Random data
        # } YHSM_RANDOM_GENERATE_RESP;
        num_bytes = pyhsm.util.validate_cmd_response_int('num_bytes', ord(data[0]), self.num_bytes)
        return data[1:1 + num_bytes]


class YHSM_Cmd_Random_Reseed(YHSM_Cmd):
    """
    Provide YubiHSM DRBG_CTR with a new seed.
    """

    status = None

    def __init__(self, stick, seed):
        seed = pyhsm.util.input_validate_str(seed, 'seed', exact_len = pyhsm.defines.YSM_CTR_DRBG_SEED_SIZE)
        # #define YSM_CTR_DRBG_SEED_SIZE      32
        # typedef struct {
        #   uint8_t seed[YSM_CTR_DRBG_SEED_SIZE];   // New seed
        # } YSM_RANDOM_RESEED_REQ;
        fmt = "%is" % (pyhsm.defines.YSM_CTR_DRBG_SEED_SIZE)
        packed = struct.pack(fmt, seed)
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_RANDOM_RESEED, packed)

    def parse_result(self, data):
        # typedef struct {
        #   YSM_STATUS status;                  // Status
        # } YSM_RANDOM_RESEED_RESP;
        fmt = "B"
        self.status, = struct.unpack(fmt, data)
        if self.status == pyhsm.defines.YSM_STATUS_OK:
            return True
        else:
            raise pyhsm.exception.YHSM_CommandFailed(pyhsm.defines.cmd2str(self.command), self.status)


class YHSM_Cmd_Temp_Key_Load(YHSM_Cmd):
    """
    Load an AEAD into the phantom key handle 0xffffffff.

    The `aead' is either a YHSM_GeneratedAEAD, or a string.
    """

    status = None

    def __init__(self, stick, nonce, key_handle, aead):
        self.nonce = pyhsm.util.input_validate_nonce(nonce, pad = True)
        self.key_handle = pyhsm.util.input_validate_key_handle(key_handle)
        flags_size = struct.calcsize("<I")
        max_aead_len = pyhsm.defines.YSM_MAX_KEY_SIZE + flags_size + pyhsm.defines.YSM_AEAD_MAC_SIZE
        aead = pyhsm.util.input_validate_aead(aead, max_aead_len = max_aead_len)
        # typedef struct {
        #   uint8_t nonce[YSM_AEAD_NONCE_SIZE]; // Nonce
        #   uint32_t keyHandle;                 // Key handle to unlock AEAD
        #   uint8_t numBytes;                   // Number of bytes (explicit key size
        #                                       //   16, 20, 24 or 32 bytes + flags + hash)
        #   uint8_t aead[YSM_MAX_KEY_SIZE + sizeof(uint32_t) + YSM_AEAD_MAC_SIZE]; // AEAD block
        # } YSM_TEMP_KEY_LOAD_REQ;
        fmt = "< %is I B %is" % (pyhsm.defines.YSM_AEAD_NONCE_SIZE, len(aead))
        packed = struct.pack(fmt, self.nonce, self.key_handle, len(aead), aead)
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_TEMP_KEY_LOAD, packed)

    def parse_result(self, data):
        """
        Parse result of L{pyhsm.defines.YSM_TEMP_KEY_LOAD} command.

        @return: Only returns (True) on successful load
        @rtype: bool

        @raise pyhsm.exception.YHSM_CommandFailed: YubiHSM failed to load key
        """
        # typedef struct {
        #   uint8_t nonce[YSM_AEAD_NONCE_SIZE]; // Nonce
        #   uint32_t keyHandle;                 // Key handle
        #   YSM_STATUS status;                  // Status
        # } YSM_TEMP_KEY_LOAD_RESP;
        fmt = "< %is I B" % (pyhsm.defines.YSM_AEAD_NONCE_SIZE)
        nonce, key_handle, self.status = struct.unpack(fmt, data)

        # Validate data in response against values we used in request
        pyhsm.util.validate_cmd_response_str('nonce', nonce, self.nonce)
        pyhsm.util.validate_cmd_response_hex('key_handle', key_handle, self.key_handle)

        if self.status == pyhsm.defines.YSM_STATUS_OK:
            return True
        else:
            raise pyhsm.exception.YHSM_CommandFailed(pyhsm.defines.cmd2str(self.command), self.status)


class YHSM_Cmd_Nonce_Get(YHSM_Cmd):
    """
    Get nonce value from YubiHSM - causes it to increment by one (or a specified number).

    Call with post_increment 0 to just fetch current value.
    """

    status = None
    response = None

    def __init__(self, stick, post_increment):
        pyhsm.util.input_validate_int(post_increment, 'post_increment')
        # typedef struct {
        #   uint16_t postIncrement;                 // Size of increment to next nonce
        # } YSM_NONCE_GET_REQ;
        packed = struct.pack("<H", post_increment)
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_NONCE_GET, packed)

    def parse_result(self, data):
        # typedef struct {
        #   YSM_STATUS status;                  // Status
        #   uint8_t nonce[YSM_AEAD_NONCE_SIZE]; // Nonce
        # } YSM_NONCE_GET_RESP;
        fmt = "B %is" % (pyhsm.defines.YSM_AEAD_NONCE_SIZE)
        self.status, nonce = struct.unpack(fmt, data)
        if self.status == pyhsm.defines.YSM_STATUS_OK:
            self.response = YHSM_NonceResponse(nonce)
            return self.response
        else:
            raise pyhsm.exception.YHSM_CommandFailed(pyhsm.defines.cmd2str(self.command), self.status)

class YHSM_Cmd_Key_Storage_Unlock(YHSM_Cmd):
    """
    Have the YubiHSM unlock it's key storage using the HSM password.

    If an incorrect password is given when the key storage is unlocked,
    it will be locked again.

    This command was replaced by YHSM_Cmd_Key_Store_Decrypt in YubiHSM 1.0.

    @ivar status: The result of the unlock operation
    @type status: integer
    """

    status = None

    def __init__(self, stick, password=''):
        payload = pyhsm.util.input_validate_str(password, 'password', max_len = pyhsm.defines.YSM_BLOCK_SIZE)
        # typedef struct {
        #   uint8_t password[YSM_BLOCK_SIZE];  // Unlock password
        # } YSM_KEY_STORAGE_UNLOCK_REQ;
        packed = payload.ljust(pyhsm.defines.YSM_BLOCK_SIZE, chr(0x0))
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_KEY_STORAGE_UNLOCK, packed)

    def parse_result(self, data):
        """
        Parse result of L{pyhsm.defines.YSM_KEY_STORAGE_UNLOCK} command.

        @return: Only returns (True) on successful unlock
        @rtype: bool

        @raise pyhsm.exception.YHSM_CommandFailed: YubiHSM failed to unlock key storage
        """
        # typedef struct {
        #   YSM_STATUS status;                  // Unlock status
        # } YSM_KEY_STORAGE_UNLOCK_RESP;
        fmt = "B"
        self.status, = struct.unpack(fmt, data)

        if self.status == pyhsm.defines.YSM_STATUS_OK:
            return True
        else:
            raise pyhsm.exception.YHSM_CommandFailed(pyhsm.defines.cmd2str(self.command), self.status)

class YHSM_Cmd_Key_Store_Decrypt(YHSM_Cmd):
    """
    Have the YubiHSM decrypt it's key store using the master key.

    If an incorrect master key is given, the key store will become unavailable
    (cleared from RAM in YubiHSM).

    @ivar status: The result of the decrypt operation
    @type status: integer
    """

    status = None

    def __init__(self, stick, key=''):
        payload = pyhsm.util.input_validate_str(key, 'key', max_len = pyhsm.defines.YSM_MAX_KEY_SIZE)
        # typedef struct {
        #   uint8_t key[YSM_MAX_KEY_SIZE];      // Key store decryption key
        # } YSM_KEY_STORE_DECRYPT_REQ;
        packed = payload.ljust(pyhsm.defines.YSM_MAX_KEY_SIZE, chr(0x0))
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_KEY_STORE_DECRYPT, packed)

    def parse_result(self, data):
        """
        Parse result of L{pyhsm.defines.YSM_KEY_STORE_DECRYPT} command.

        @return: Only returns (True) on successful decrypt
        @rtype: bool

        @raise pyhsm.exception.YHSM_CommandFailed: YubiHSM failed to decrypt key store
        """
        # typedef struct {
        #   YSM_STATUS status;                  // Decrypt status
        # } YSM_KEY_STORE_DECRYPT_RESP;
        fmt = "B"
        self.status, = struct.unpack(fmt, data)

        if self.status == pyhsm.defines.YSM_STATUS_OK:
            return True
        else:
            raise pyhsm.exception.YHSM_CommandFailed(pyhsm.defines.cmd2str(self.command), self.status)

class YHSM_Cmd_HSM_Unlock(YHSM_Cmd):
    """
    Have the YubiHSM unlock the HSM operations (those involving the keystore) with a YubiKey OTP.

    Admin YubiKeys public id's are entered during initial configuration. These must then be made
    available in the internal database.

    @ivar status: The result of the unlock operation
    @type status: integer
    """

    status = None

    def __init__(self, stick, public_id, otp):
        self.public_id = pyhsm.util.input_validate_nonce(public_id, pad = True)
        self.otp = pyhsm.util.input_validate_str(otp, 'otp', exact_len = pyhsm.defines.YSM_OTP_SIZE)
        # typedef struct {
        #   uint8_t publicId[YSM_PUBLIC_ID_SIZE]; // Public id
        #   uint8_t otp[YSM_OTP_SIZE];          // OTP
        # } YSM_HSM_UNLOCK_REQ;
        fmt = "< %is %is" % (pyhsm.defines.YSM_AEAD_NONCE_SIZE, \
                                       pyhsm.defines.YSM_OTP_SIZE, \
                                 )
        packed = struct.pack(fmt, self.public_id, \
                                 self.otp,
                             )
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_HSM_UNLOCK, packed)

    def parse_result(self, data):
        """
        Parse result of L{pyhsm.defines.YSM_HSM_UNLOCK} command.

        @return: Only returns (True) on successful unlock
        @rtype: bool

        @raise pyhsm.exception.YHSM_CommandFailed: YubiHSM failed to unlock key store
        """
        # typedef struct {
        #   YSM_STATUS status;                  // Unlock status
        # } YSM_HSM_UNLOCK_RESP;
        fmt = "B"
        self.status, = struct.unpack(fmt, data)

        if self.status == pyhsm.defines.YSM_STATUS_OK:
            return True
        else:
            raise pyhsm.exception.YHSM_CommandFailed(pyhsm.defines.cmd2str(self.command), self.status)

class YHSM_NonceResponse():
    """ Small class to hold response of Nonce_Get command.

    @ivar volatile: Volatile part of nonce
    @ivar pu_count: Power-up count -- persistent part of nonce
    @ivar nonce_int: Current nonce
    @ivar nonce: Current nonce

    @type volatile: integer
    @type pu_count: integer
    @type nonce_int: long
    @type nonce: string
    """
    volatile = 0
    pu_count = 0
    nonce_int = 0
    nonce = None

    def __init__(self, nonce):
        # The power-up count can be deduced from the nonce =)
        self.volatile = struct.unpack("<L", nonce[0:4])[0]
        self.pu_count = struct.unpack("<H", nonce[4:6])[0]
        self.nonce_int = (self.pu_count << 32) + self.volatile
        self.nonce = nonce

    def __repr__(self):
        return '<%s instance at %s: nonce=%s, pu_count=%i, volatile=%i>' % (
            self.__class__.__name__,
            hex(id(self)),
            self.nonce.encode('hex'),
            self.pu_count,
            self.volatile
            )



"""
implementations of internal buffer commands for YubiHSM
"""

# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import struct

__all__ = [
    # constants
    # functions
    # classes
    'YHSM_Cmd_Buffer_Load',
    'YHSM_Cmd_Buffer_Random_Load',
]

import pyhsm.defines
import pyhsm.exception
from pyhsm.cmd import YHSM_Cmd

class YHSM_Cmd_Buffer_Load(YHSM_Cmd):
    """
    Ask YubiHSM to load some data into it's internal buffer.

    @ivar data_len: Number of bytes (to be) loaded
    @ivar offset: Offset in internal buffer where data will be/was loaded

    @type data_len: integer
    @type offset: integer
    """

    data_len = 0
    offest = 0

    def __init__(self, stick, data, offset = 0):
        data = pyhsm.util.input_validate_str(data, 'data', max_len = pyhsm.defines.YSM_DATA_BUF_SIZE)
        self.data_len = len(data)
        self.offset = pyhsm.util.input_validate_int(offset, 'offset', pyhsm.defines.YSM_DATA_BUF_SIZE - 1)
        # typedef struct {
        #   uint8_t offs;                       // Offset in buffer. Zero flushes/resets buffer first
        #   uint8_t numBytes;                   // Number of bytes to load
        #   uint8_t data[YSM_DATA_BUF_SIZE];    // Data to load
        # } YSM_BUFFER_LOAD_REQ;
        fmt = "B B %is" % self.data_len
        packed = struct.pack(fmt, self.offset, self.data_len, data)
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_BUFFER_LOAD, packed)

    def parse_result(self, data):
        """
        Parse result of L{pyhsm.defines.YSM_BUFFER_LOAD} command.

        @return: Number of bytes now in the YubiHSM internal buffer.
        @rtype: integer

        @raise pyhsm.exception.YHSM_Error: Unexpected number of bytes were loaded
        """
        # typedef struct {
        #   uint8_t numBytes;                   // Number of bytes in buffer now
        # } YSM_BUFFER_LOAD_RESP;
        count = ord(data[0])
        if self.offset == 0:
            # if offset was 0, the buffer was reset and
            # we can verify the length returned
            if count != self.data_len:
                raise pyhsm.exception.YHSM_Error("Incorrect number of bytes in buffer (got %i, expected %i)" \
                                               % (self.data_len, count))
        return count

class YHSM_Cmd_Buffer_Random_Load(YHSM_Cmd):
    """
    Ask YubiHSM to generate a secret for a specific public_id

    Generated secret is stored in YubiHSM's internal memory and is
    retreived using YHSM_Cmd_Blob_Generate.
    """
    def __init__(self, stick, num_bytes, offset = 0):
        self.offset = offset
        self.num_bytes = num_bytes
        # typedef struct {
        #   uint8_t offs;                       // Offset in buffer. Zero flushes/resets buffer first
        #   uint8_t numBytes;                   // Number of bytes to randomize
        # } YSM_BUFFER_RANDOM_LOAD_REQ;
        fmt = "B B"
        packed = struct.pack(fmt, self.offset, self.num_bytes)
        YHSM_Cmd.__init__(self, stick, pyhsm.defines.YSM_BUFFER_RANDOM_LOAD, packed)

    def parse_result(self, data):
        """ Return True if the public_id in the response matches the one in the request. """
        # typedef struct {
        #   uint8_t numBytes;                   // Number of bytes in buffer now
        # } YSM_BUFFER_LOAD_RESP;
        count = ord(data[0])
        if self.offset == 0:
            # if offset was 0, the buffer was reset and
            # we can verify the length returned
            if count != self.num_bytes:
                raise pyhsm.exception.YHSM_Error("Incorrect number of bytes in buffer (got %i, expected %i)" \
                                               % (self.num_bytes, count))
        return count



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import os
import sys
import unittest
import pyhsm
import struct
from pyhsm.soft_hsm import crc16

# configuration parameters
CfgPassphrase = ""
HsmPassphrase = "bada" * 2
PrimaryAdminYubiKey = ('ftftftfteeee', 'f0f1f2f3f4f5', '4d' * 16,)
AdminYubiKeys = [PrimaryAdminYubiKey[0]]

class YHSM_TestCase(unittest.TestCase):

    hsm = None

    def setUp(self, device = os.getenv('YHSM_DEVICE', '/dev/ttyACM0'), debug = False):
        """
        Common initialization class for our tests. Initializes a
        YubiHSM in self.hsm.
        """
        self.hsm = pyhsm.base.YHSM(device = device, debug = debug)
        # unlock keystore if our test configuration contains a passphrase
        if HsmPassphrase is not None and HsmPassphrase != "":
            try:
                self.hsm.unlock(password = HsmPassphrase.decode("hex"))
                self.otp_unlock()
            except pyhsm.exception.YHSM_CommandFailed, e:
                # ignore errors from the unlock function, in case our test configuration
                # hasn't been loaded into the YubiHSM yet
                pass

    def tearDown(self):
        # get destructor called properly
        self.hsm = None

    def who_can(self, what, expected = [], extra_khs = []):
        """
        Try the lambda what() with all key handles between 1 and 32, except the expected one.
        Fail on anything but YSM_FUNCTION_DISABLED.
        """
        for kh in list(xrange(1, 32)) + extra_khs:
            if kh in expected:
                continue
            res = None
            try:
                res = what(kh)
                self.fail("Expected YSM_FUNCTION_DISABLED for key handle 0x%0x, got '%s'" % (kh, res))
            except pyhsm.exception.YHSM_CommandFailed, e:
                if e.status != pyhsm.defines.YSM_FUNCTION_DISABLED:
                    self.fail("Expected YSM_FUNCTION_DISABLED for key handle 0x%0x, got %s" \
                                  % (kh, e.status_str))

    def otp_unlock(self):
        """
        Do OTP unlock of the YubiHSM keystore.

        Since we don't always reprogram the YubiHSM, we might need to hunt for an unused OTP.
        """
        if not self.hsm.version.have_unlock():
            return None
        Params = PrimaryAdminYubiKey
        YK = FakeYubiKey(pyhsm.yubikey.modhex_decode(Params[0]).decode('hex'),
                         Params[1].decode('hex'), Params[2].decode('hex')
                         )
        YK.session_ctr = 0
        use_ctr = 1	# the 16 bit power-up counter of the YubiKey
        while use_ctr < 0xffff:
            YK.use_ctr = use_ctr
            otp = YK.from_key()
            try:
                res = self.hsm.unlock(otp = otp)
                self.assertTrue(res)
                # OK - if we got here we've got a successful response for this OTP
                break
            except pyhsm.exception.YHSM_CommandFailed, e:
                if e.status != pyhsm.defines.YSM_OTP_REPLAY:
                    raise
            # don't bother with the session_ctr - test run 5 would mean we first have to
            # exhaust 4 * 256 session_ctr increases before the YubiHSM would pass our OTP
            use_ctr += 1

class YubiKeyEmu():
    """
    Emulate the internal memory of a YubiKey.
    """

    def __init__(self, user_id, use_ctr, timestamp, session_ctr):
        if len(user_id) != pyhsm.defines.UID_SIZE:
            raise pyhsm.exception.YHSM_WrongInputSize(
                'user_id', pyhsm.defines.UID_SIZE, len(user_id))

        self.user_id = user_id
        self.use_ctr = use_ctr
        self.timestamp = timestamp
        self.session_ctr = session_ctr
        self.rnd = struct.unpack('H', os.urandom(2))[0]

    def pack(self):
        """
        Return contents packed. Only add AES ECB encryption and modhex to
        get your own YubiKey OTP (see function 'from_key').
        """

        #define UID_SIZE 6
	#typedef struct {
        #  uint8_t userId[UID_SIZE];
        #  uint16_t sessionCtr;		# NOTE: this is use_ctr
        #  uint24_t timestamp;
        #  uint8_t sessionUse;		# NOTE: this is session_ctr
        #  uint16_t rnd;
        #  uint16_t crc;
	#} TICKET;
        fmt = "< %is H HB B H" % (pyhsm.defines.UID_SIZE)

        ts_high = (self.timestamp & 0x00ff0000) >> 16
        ts_low  =  self.timestamp & 0x0000ffff

        res = struct.pack(fmt, self.user_id, \
                              self.use_ctr, \
                              ts_low, ts_high, \
                              self.session_ctr, \
                              self.rnd)
        crc = 0xffff - crc16(res)

        return res + struct.pack('<H', crc)

    def get_otp(self, key):
        """
        Return an modhex encoded OTP given our current state.
        """
        from Crypto.Cipher import AES
        packed = self.pack()
        obj = AES.new(key, AES.MODE_ECB)
        ciphertext = obj.encrypt(packed)
        return ciphertext

    def from_key(self, public_id, key):
        """
        Return what the YubiKey would have returned when the button was pressed.
        """
        from pyhsm.yubikey import modhex_encode, modhex_decode

        otp = self.get_otp(key)
        from_key = modhex_encode(public_id.encode('hex')) + modhex_encode(otp.encode('hex'))
        return from_key

class YubiKeyRnd(YubiKeyEmu):
    """ YubiKeyEmu with everything but user_id randomized. """

    def __init__(self, user_id):
        timestamp, session_counter, session_use = struct.unpack('IHB', os.urandom(7))
        YubiKeyEmu.__init__(self, user_id, session_counter, timestamp, session_use)

class FakeYubiKey(YubiKeyEmu):
    """
    A complete fake YubiKey.
    """

    def __init__(self, public_id, user_id, key):
        use_ctr, timestamp, session_ctr, = (0, 0, 0,)
        YubiKeyEmu.__init__(self, user_id, use_ctr, timestamp, session_ctr)
        self.public_id = public_id
        self.key = key

    def get_otp(self, key = None):
        if key is None:
            key = self.key
        return YubiKeyEmu.get_otp(self, key)

    def from_key(self):
        return YubiKeyEmu.from_key(self, self.public_id, self.key)



# Copyright (c) 2013 Yubico AB
# All rights reserved.
#
#   Redistribution and use in source and binary forms, with or
#   without modification, are permitted provided that the following
#   conditions are met:
#
#    1. Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from __future__ import absolute_import

from setuptools import Command
from distutils.errors import DistutilsSetupError
import os
import json
import tempfile


class executable(Command):
    description = "create an executable"
    user_options = [
        ('debug', None, "build with debug flag"),
        ('data-files=', None, "data files to include"),
        ('package-version=', None, "package version")
    ]
    boolean_options = ['debug']

    def initialize_options(self):
        self.debug = 0
        self.data_files = ''
        self.package_version = '0'

    def finalize_options(self):
        self.cwd = os.getcwd()
        self.data_files = self.data_files.split()
        self.package_version = int(self.package_version)

    def run(self):
        if os.getcwd() != self.cwd:
            raise DistutilsSetupError("Must be in package root!")

        from PyInstaller.main import run as pyinst_run

        os.environ['pyinstaller_data'] = json.dumps({
            'debug': self.debug,
            'name': self.distribution.get_name(),
            'long_name': os.environ['setup_long_name'],
            'data_files': self.data_files,
            'package_version': self.package_version
        })

        spec = tempfile.NamedTemporaryFile(suffix='.spec', delete=False)
        source = os.path.join(os.path.dirname(__file__), 'pyinstaller_spec.py')
        with open(source) as f:
            spec.write(f.read())
        spec_name = spec.name
        spec.close()
        pyinst_run([spec_name])
        os.unlink(spec_name)

        self.announce("Executable created!")



"""
module for accessing a YubiHSM
"""

# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import re
import struct

__all__ = [
    # constants
    # functions
    'reset',
    # classes
    'YHSM_Cmd',
]

import pyhsm.exception
import pyhsm.defines

class YHSM_Cmd():
    """
    Base class for YubiHSM commands.
    """

    response_status = None
    executed = False

    def __init__(self, stick, command, payload=''):
        """
        The base class for all YSM_ commands.

        @param stick: Reference to a YubiHSM
        @param command: The YSM_xxx command defined in pyhsm.defines.
        @param payload: a packed C struct, represented as a Python string

        @type stick: L{pyhsm.stick.YHSM_Stick}
        @type command: integer
        @type payload: string
        """
        self.stick = stick
        self.command = command
        self.payload = payload
        return None

    def execute(self, read_response=True):
        """
        Write command to HSM and read response.

        @param read_response: Whether to expect a response or not.
        @type read_response: bool
        """
        # // Up- and downlink packet
        # typedef struct {
        #   uint8_t bcnt;                       // Number of bytes (cmd + payload)
        #   uint8_t cmd;                        // YSM_xxx command
        #   uint8_t payload[YSM_MAX_PKT_SIZE];  // Payload
        # } YSM_PKT;
        if self.command != pyhsm.defines.YSM_NULL:
            # YSM_NULL is the exception to the rule - it should NOT be prefixed with YSM_PKT.bcnt
            cmd_buf = struct.pack('BB', len(self.payload) + 1, self.command)
        else:
            cmd_buf = chr(self.command)
        cmd_buf += self.payload
        debug_info = None
        unlock = self.stick.acquire()
        try:
            if self.stick.debug:
                debug_info = "%s (payload %i/0x%x)" % (pyhsm.defines.cmd2str(self.command), \
                                                        len(self.payload), len(self.payload))
            self.stick.write(cmd_buf, debug_info)
            if not read_response:
                return None
            return self._read_response()
        finally:
            unlock()

    def _read_response(self):
        """
        After writing a command, read response.

        @returns: Result of parse_data()

        @raises pyhsm.exception.YHSM_Error: On failure to read a response to the
            command we sent in a timely fashion.
        """
        # // Up- and downlink packet
        # typedef struct {
        #   uint8_t bcnt;                       // Number of bytes (cmd + payload)
        #   uint8_t cmd;                        // YSM_xxx command
        #   uint8_t payload[YSM_MAX_PKT_SIZE];  // Payload
        # } YSM_PKT;

        # read YSM_PKT.bcnt and YSM_PKT.cmd
        res = self.stick.read(2, 'response length + response status')
        if len(res) != 2:
            self._handle_invalid_read_response(res, 2)
        response_len, response_status = struct.unpack('BB', res)
        response_len -= 1 # the status byte has been read already
        debug_info = None
        if response_status & pyhsm.defines.YSM_RESPONSE:
            debug_info = "%s response (%i/0x%x bytes)" \
                % (pyhsm.defines.cmd2str(response_status - pyhsm.defines.YSM_RESPONSE), \
                       response_len, response_len)
        # read YSM_PKT.payload
        res = self.stick.read(response_len, debug_info)
        if res:
            if response_status == self.command | pyhsm.defines.YSM_RESPONSE:
                self.executed = True
                self.response_status = response_status
                return self.parse_result(res)
            else:
                reset(self.stick)
                raise pyhsm.exception.YHSM_Error('YubiHSM responded to wrong command')
        else:
            raise pyhsm.exception.YHSM_Error('YubiHSM did not respond')

    def _handle_invalid_read_response(self, res, expected_len):
        """
        This function is called when we do not get the expected frame header in
        response to a command. Probable reason is that we are not talking to a
        YubiHSM in HSM mode (might be a modem, or a YubiHSM in configuration mode).

        Throws a hopefully helpful exception.
        """
        if not res:
            reset(self.stick)
            raise pyhsm.exception.YHSM_Error('YubiHSM did not respond to command %s' \
                                                 % (pyhsm.defines.cmd2str(self.command)) )
        # try to check if it is a YubiHSM in configuration mode
        self.stick.write('\r\r\r', '(mode test)')
        res2 = self.stick.read(50) # expect a timeout
        lines = res2.split('\n')
        for this in lines:
            if re.match('^(NO_CFG|WSAPI|HSM).*> .*', this):
                raise pyhsm.exception.YHSM_Error('YubiHSM is in configuration mode')
        raise pyhsm.exception.YHSM_Error('Unknown response from serial device %s : "%s"' \
                                             % (self.stick.device, res.encode('hex')))

    def parse_result(self, data):
        """
        This function is intended to be overridden by sub-classes that
        implements commands that should not just return the data read from
        the YubiHSM.
        """
        return data

def reset(stick):
    """
    Send a bunch of zero-bytes to the YubiHSM, and flush the input buffer.
    """
    nulls = (pyhsm.defines.YSM_MAX_PKT_SIZE - 1) * '\x00'
    res = YHSM_Cmd(stick, pyhsm.defines.YSM_NULL, payload = nulls).execute(read_response = False)
    unlock = stick.acquire()
    try:
        stick.drain()
        stick.flush()
    finally:
        unlock()
    return res == 0



"""
common exceptions for the pyhsm package
"""

# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

__all__ = [
    # constants
    # functions
    # classes
    'YHSM_Error',
    'YHSM_InputTooShort',
    'YHSM_InputTooLong',
    'YHSM_WrongInputSize',
    'YHSM_WrongInputType',
    'YHSM_CommandFailed'
]

import pyhsm.defines

class YHSM_Error(Exception):
    """
    Base class for YHSM exceptions in this package.

    @ivar reason: explanation of the error
    @type reason: string
    """

    reason = None

    def __init__(self, reason):
        Exception.__init__(self)
        self.reason = reason

    def __str__(self):
        return '<%s instance at %s: %s>' % (
            self.__class__.__name__,
            hex(id(self)),
            self.reason
            )

class YHSM_WrongInputSize(YHSM_Error):
    """
    Exception raised for errors in the size of an argument to some function.
    """
    def __init__(self, name, expected, size):
        reason = "Bad size of argument '%s', expected %i got %i" % (name, expected, size)
        YHSM_Error.__init__(self, reason)

class YHSM_InputTooShort(YHSM_Error):
    """
    Exception raised for too short input to some function.
    """
    def __init__(self, name, expected, size):
        reason = "Argument '%s' too short, expected min %i got %i" % (name, expected, size)
        YHSM_Error.__init__(self, reason)

class YHSM_InputTooLong(YHSM_Error):
    """
    Exception raised for too long input to some function.
    """
    def __init__(self, name, expected, size):
        reason = "Argument '%s' too long, expected max %i got %i" % (name, expected, size)
        YHSM_Error.__init__(self, reason)

class YHSM_WrongInputType(YHSM_Error):
    """
    Exception raised for errors in the type of an argument to some function.
    """
    def __init__(self, name, expected, name_type):
        reason = "Bad type of argument '%s', expected %s got %s" % (name, expected, name_type)
        YHSM_Error.__init__(self, reason)

class YHSM_CommandFailed(YHSM_Error):
    """
    Exception raised when a command sent to the YubiHSM returned an error.
    """
    def __init__(self, name, status):
        self.status = status
        self.status_str = pyhsm.defines.status2str(status)
        reason = "Command %s failed: %s" % (name, self.status_str)
        YHSM_Error.__init__(self, reason)



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import sys
import unittest
import pyhsm

import test_common

class TestUtil(test_common.YHSM_TestCase):

    def setUp(self):
        test_common.YHSM_TestCase.setUp(self)

    def test_hexdump(self):
        """ Test hexdump function. """
        data1 = ''.join([chr(x) for x in xrange(8)])
        self.assertEquals('0000   00 01 02 03 04 05 06 07\n', pyhsm.util.hexdump(data1))
        data2 = ''.join([chr(x) for x in xrange(64)])
        self.assertEquals(248, len(pyhsm.util.hexdump(data2)))
        self.assertEquals('', pyhsm.util.hexdump(''))

    def test_response_validation(self):
        """ Test response validation functions. """
        self.assertRaises(pyhsm.exception.YHSM_Error, pyhsm.util.validate_cmd_response_str, \
                              'test', 'abc', 'def', hex_encode=True)

        self.assertRaises(pyhsm.exception.YHSM_Error, pyhsm.util.validate_cmd_response_str, \
                              'test', 'abc', 'def', hex_encode=False)

    def test_input_validate_str(self):
        """ Test string input validation. """
        self.assertRaises(pyhsm.exception.YHSM_WrongInputType, pyhsm.util.input_validate_str, \
                              0, 'foo', exact_len = 5)

        self.assertRaises(pyhsm.exception.YHSM_InputTooLong, pyhsm.util.input_validate_str, \
                              '1234', 'foo', max_len = 3)
        self.assertEquals('1234', pyhsm.util.input_validate_str('1234', 'foo', max_len = 4))
        self.assertEquals('1234', pyhsm.util.input_validate_str('1234', 'foo', max_len = 14))

        self.assertRaises(pyhsm.exception.YHSM_WrongInputSize, pyhsm.util.input_validate_str, \
                              '1234', 'foo', exact_len = 5)
        self.assertEquals('1234', pyhsm.util.input_validate_str('1234', 'foo', exact_len = 4))



"""
implementations of validation commands for YubiHSM
"""

# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import struct

__all__ = [
    # constants
    # functions
    # classes
    'YHSM_Cmd_AEAD_Validate_OTP',
    'YHSM_ValidationResult',
]

import pyhsm.defines
import pyhsm.exception
from pyhsm.aead_cmd import YHSM_AEAD_Cmd

class YHSM_Cmd_AEAD_Validate_OTP(YHSM_AEAD_Cmd):
    """
    Request the YubiHSM to validate an OTP using an externally stored AEAD.
    """

    response = None
    status = None

    def __init__(self, stick, public_id, otp, key_handle, aead):
        self.public_id = pyhsm.util.input_validate_nonce(public_id, pad = True)
        self.otp = pyhsm.util.input_validate_str(otp, 'otp', exact_len = pyhsm.defines.YSM_OTP_SIZE)
        self.key_handle = pyhsm.util.input_validate_key_handle(key_handle)
        aead = pyhsm.util.input_validate_aead(aead, expected_len = pyhsm.defines.YSM_YUBIKEY_AEAD_SIZE)
        # typedef struct {
        #   uint8_t publicId[YSM_PUBLIC_ID_SIZE];   // Public id (nonce)
        #   uint32_t keyHandle;                     // Key handle
        #   uint8_t otp[YSM_OTP_SIZE];              // OTP
        #   uint8_t aead[YSM_YUBIKEY_AEAD_SIZE];    // AEAD block
        # } YSM_AEAD_YUBIKEY_OTP_DECODE_REQ;
        fmt = "< %is I %is %is" % (pyhsm.defines.YSM_AEAD_NONCE_SIZE, \
                                       pyhsm.defines.YSM_OTP_SIZE, \
                                       pyhsm.defines.YSM_YUBIKEY_AEAD_SIZE)
        packed = struct.pack(fmt, self.public_id, \
                                 self.key_handle, \
                                 self.otp, \
                                 aead)
        YHSM_AEAD_Cmd.__init__(self, stick, pyhsm.defines.YSM_AEAD_YUBIKEY_OTP_DECODE, packed)

    def parse_result(self, data):
        # typedef struct {
        #   uint8_t public_id[YSM_PUBLIC_ID_SIZE];   // Public id
        #   uint32_t keyHandle;                  // Key handle
        #   uint16_t use_ctr;                    // Use counter
        #   uint8_t session_ctr;                 // Session counter
        #   uint8_t tstph;                       // Timestamp (high part)
        #   uint16_t tstpl;                      // Timestamp (low part)
        #   YHSM_STATUS status;                  // Validation status
        # } YHSM_AEAD_OTP_DECODED_RESP;
        fmt = "< %is I H B B H B" % (pyhsm.defines.YSM_PUBLIC_ID_SIZE)
        public_id, \
            key_handle, \
            use_ctr, \
            session_ctr, \
            ts_high, \
            ts_low, \
            self.status = struct.unpack(fmt, data)

        pyhsm.util.validate_cmd_response_str('public_id', public_id, self.public_id)
        pyhsm.util.validate_cmd_response_hex('key_handle', key_handle, self.key_handle)

        if self.status == pyhsm.defines.YSM_STATUS_OK:
            self.response = YHSM_ValidationResult(self.public_id, use_ctr, session_ctr, ts_high, ts_low)
            return self.response
        else:
            raise pyhsm.exception.YHSM_CommandFailed(pyhsm.defines.cmd2str(self.command), self.status)


class YHSM_ValidationResult():
    """
    The result of a Validate operation.

    Contains the counters and timestamps decrypted from the OTP.

    @ivar public_id: The six bytes public ID of the YubiKey that produced the OTP
    @ivar use_ctr: The 16-bit power-on non-volatile counter of the YubiKey
    @ivar session_ctr: The 8-bit volatile session counter of the YubiKey
    @ivar ts_high: The high 8 bits of the 24-bit 8 hz timer since power-on of the YubiKey
    @ivar ts_low: The low 16 bits of the 24-bit 8 hz timer since power-on of the YubiKey
    @type public_id: string
    @type use_ctr: integer
    @type session_ctr: integer
    @type ts_high: integer
    @type ts_low: integer
    """

    public_id = use_ctr = session_ctr = ts_high = ts_low = None

    def __init__(self, public_id, use_ctr, session_ctr, ts_high, ts_low):
        self.public_id = public_id
        self.use_ctr = use_ctr
        self.session_ctr = session_ctr
        self.ts_high = ts_high
        self.ts_low = ts_low

    def __repr__(self):
        return '<%s instance at %s: public_id=%s, use_ctr=%i, session_ctr=%i, ts=%i/%i>' % (
            self.__class__.__name__,
            hex(id(self)),
            self.public_id.encode('hex'),
            self.use_ctr,
            self.session_ctr,
            self.ts_high,
            self.ts_low
            )



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import os
import sys
import unittest
import pyhsm

import test_aead
import test_aes_ecb
import test_basics
import test_buffer
import test_db
import test_hmac
import test_oath
import test_otp_validate
import test_stick
import test_util
import test_yubikey_validate
import test_misc
import test_soft_hsm

test_modules = [test_aead,
                test_aes_ecb,
                test_basics,
                test_buffer,
                test_db,
                test_hmac,
                test_oath,
                test_otp_validate,
                test_stick,
                test_util,
                test_yubikey_validate,
                test_misc,
                test_soft_hsm,
                ]

# special, should not be addded to test_modules
import test_configure

def suite():
    """
    Create a test suite with all our tests.

    If the OS environment variable 'YHSM_ZAP' is set and evaluates to true,
    we will include the special test case class that erases the current
    YubiHSM config and creates a new one with known keys to be used by the
    other tests. NOTE that this is ONLY POSSIBLE if the YubiHSM is already
    in DEBUG mode.
    """

    global test_modules

    # Check if we have a YubiHSM present, and start with locking it's keystore
    # XXX produce a better error message than 'error: None' when initializing fails
    hsm = pyhsm.YHSM(device = os.getenv('YHSM_DEVICE', '/dev/ttyACM0'))
    try:
        hsm.unlock("BADPASSPHRASE99")
    except pyhsm.exception.YHSM_CommandFailed, e:
        if hsm.version.have_key_store_decrypt():
            if e.status != pyhsm.defines.YSM_MISMATCH:
                raise
        else:
            if e.status != pyhsm.defines.YSM_KEY_STORAGE_LOCKED and \
                    e.status != pyhsm.defines.YSM_FUNCTION_DISABLED:
                raise

    zap = []
    if 'YHSM_ZAP' in os.environ:
        if os.environ['YHSM_ZAP']:
            zap = [unittest.TestLoader().loadTestsFromModule(test_configure)]

    l = zap + [unittest.TestLoader().loadTestsFromModule(this) for this in test_modules]

    return unittest.TestSuite(l)

if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

import sys
import string
import unittest
import pyhsm

import test_common
from test_common import YubiKeyEmu, YubiKeyRnd

class TestYubikeyValidate(test_common.YHSM_TestCase):

    def setUp(self):
        test_common.YHSM_TestCase.setUp(self)

        self.yk_key = 'F' * 16	# 128 bit AES key
        self.yk_uid = '\x4d\x01\x4d\x02\x4d\x4d'
        self.yk_rnd = YubiKeyRnd(self.yk_uid)
        self.yk_public_id = '4d4d4d4d4d4d'.decode('hex')

        secret = pyhsm.aead_cmd.YHSM_YubiKeySecret(self.yk_key, self.yk_uid)
        self.hsm.load_secret(secret)

        # YubiHSM includes key handle id in AES-CCM of aeads, so we must use same
        # key to generate and validate. Key 0x2000 has all flags.
        self.kh_generate = 0x2000
        self.kh_validate = 0x2000

        self.aead = self.hsm.generate_aead(self.yk_public_id, self.kh_generate)

    def test_validate_aead_cmp(self):
        """ Test that the AEAD generated contains our secrets. """
        secret = pyhsm.aead_cmd.YHSM_YubiKeySecret(self.yk_key, self.yk_uid)
        cleartext = secret.pack()
        self.assertTrue(self.hsm.validate_aead(self.yk_public_id, self.kh_validate, self.aead, cleartext))
        wrong_cleartext = 'X' + cleartext[1:]
        self.assertFalse(self.hsm.validate_aead(self.yk_public_id, self.kh_validate, self.aead, wrong_cleartext))

    def test_validate_aead_cmp_long(self):
        """ Test validating a long AEAD """
        cleartext = 'C' * 36
        key_handle = 0x2000 # key 0x2000 has all flags set
        nonce = '123456'
        aead = self.hsm.generate_aead_simple(nonce, key_handle, cleartext)
        self.assertTrue(self.hsm.validate_aead(nonce, key_handle, aead, cleartext))
        wrong_cleartext = 'X' + cleartext[1:]
        self.assertFalse(self.hsm.validate_aead(nonce, key_handle, aead, wrong_cleartext))

    def test_validate_yubikey(self):
        """ Test validate YubiKey OTP. """
        from_key = self.yk_rnd.from_key(self.yk_public_id, self.yk_key)
        self.assertTrue(pyhsm.yubikey.validate_yubikey_with_aead( \
                self.hsm, from_key, self.aead, self.kh_validate))

    def test_modhex_encode_decode(self):
        """ Test modhex encoding/decoding. """
        h = '4d014d024d4ddd5382b11195144da07d'
        self.assertEquals(h, pyhsm.yubikey.modhex_decode( pyhsm.yubikey.modhex_encode(h) ) )

    def test_split_id_otp(self):
        """ Test public_id + OTP split function. """
        public_id, otp, = pyhsm.yubikey.split_id_otp("ft" * 16)
        self.assertEqual(public_id, '')
        self.assertEqual(otp, "ft" * 16)

        public_id, otp, = pyhsm.yubikey.split_id_otp("cc" + "ft" * 16)
        self.assertEqual(public_id, 'cc')
        self.assertEqual(otp, "ft" * 16)



"""
Various defines from pyhsm_if.h.
"""

# Copyright (c) 2011 Yubico AB
# See the file COPYING for licence statement.

__all__ = [
    # constants
    'YSM_PUBLIC_ID_SIZE',
    'YSM_OTP_SIZE',
    'YSM_BLOCK_SIZE',
    'YSM_MAX_KEY_SIZE',
    'YSM_DATA_BUF_SIZE',
    'YSM_AEAD_NONCE_SIZE',
    'YSM_AEAD_MAC_SIZE',
    'YSM_CCM_CTR_SIZE',
    'YSM_AEAD_MAX_SIZE',
    'YSM_SHA1_HASH_SIZE',
    'YSM_CTR_DRBG_SEED_SIZE',
    'YSM_MAX_PKT_SIZE',
    'YSM_PROTOCOL_VERSION',
    'YSM_TEMP_KEY_HANDLE',
    'UID_SIZE',
    'KEY_SIZE',
    ## statuses
    'YSM_STATUS_OK',
    'YSM_KEY_HANDLE_INVALID',
    'YSM_AEAD_INVALID',
    'YSM_OTP_INVALID',
    'YSM_OTP_REPLAY',
    'YSM_ID_DUPLICATE',
    'YSM_ID_NOT_FOUND',
    'YSM_DB_FULL',
    'YSM_MEMORY_ERROR',
    'YSM_FUNCTION_DISABLED',
    'YSM_KEY_STORAGE_LOCKED',
    'YSM_MISMATCH',
    'YSM_INVALID_PARAMETER',
    ## commands
    'YSM_NULL',
    'YSM_AEAD_GENERATE',
    'YSM_BUFFER_AEAD_GENERATE',
    'YSM_RANDOM_AEAD_GENERATE',
    'YSM_AEAD_DECRYPT_CMP',
    'YSM_DB_YUBIKEY_AEAD_STORE',
    'YSM_DB_YUBIKEY_AEAD_STORE2',
    'YSM_AEAD_YUBIKEY_OTP_DECODE',
    'YSM_DB_OTP_VALIDATE',
    'YSM_AES_ECB_BLOCK_ENCRYPT',
    'YSM_AES_ECB_BLOCK_DECRYPT',
    'YSM_AES_ECB_BLOCK_DECRYPT_CMP',
    'YSM_HMAC_SHA1_GENERATE',
    'YSM_TEMP_KEY_LOAD',
    'YSM_BUFFER_LOAD',
    'YSM_BUFFER_RANDOM_LOAD',
    'YSM_NONCE_GET',
    'YSM_ECHO',
    'YSM_RANDOM_GENERATE',
    'YSM_RANDOM_RESEED',
    'YSM_SYSTEM_INFO_QUERY',
    'YSM_MONITOR_EXIT',
    ##
    # functions
    'cmd2str',
    'status2str'
    # classes
]

YSM_PUBLIC_ID_SIZE	= 6	# Size of public id for std OTP validation
YSM_OTP_SIZE		= 16	# Size of OTP
YSM_BLOCK_SIZE		= 16	# Size of block operations
YSM_MAX_KEY_SIZE	= 32	# Max size of CCMkey
YSM_DATA_BUF_SIZE	= 64	# Size of internal data buffer
YSM_AEAD_NONCE_SIZE	= 6	# Size of AEAD nonce (excluding size of key handle)
YSM_AEAD_MAC_SIZE	= 8	# Size of AEAD MAC field
YSM_CCM_CTR_SIZE	= 2	# Sizeof of AES CCM counter field
YSM_AEAD_MAX_SIZE	= (YSM_DATA_BUF_SIZE + YSM_AEAD_MAC_SIZE) # Max size of an AEAD block
YSM_SHA1_HASH_SIZE	= 20	# 160-bit SHA1 hash size
YSM_CTR_DRBG_SEED_SIZE	= 32	# Size of CTR-DRBG entropy
YSM_MAX_PKT_SIZE	= 0x60  # Max size of a packet (excluding command byte)
YSM_PROTOCOL_VERSION	= 1	# Protocol version for this file


YSM_TEMP_KEY_HANDLE	= 0xffffffff # Phantom temporary key handle

# these two are in ykdef.h
UID_SIZE	= 6
KEY_SIZE	= 16

YSM_RESPONSE		= 0x80    # Response bit
YSM_YUBIKEY_AEAD_SIZE	= (KEY_SIZE + UID_SIZE + YSM_AEAD_MAC_SIZE)

# Response codes
YSM_STATUS_OK           = 0x80    # Executed successfully
YSM_KEY_HANDLE_INVALID  = 0x81    # Key handle is invalid
YSM_AEAD_INVALID        = 0x82    # Supplied AEAD block is invalid
YSM_OTP_INVALID         = 0x83    # Supplied OTP is invalid (CRC or UID)
YSM_OTP_REPLAY          = 0x84    # Supplied OTP is replayed
YSM_ID_DUPLICATE        = 0x85    # The supplied public ID is already in the database
YSM_ID_NOT_FOUND        = 0x86    # The supplied public ID was not found in the database
YSM_DB_FULL             = 0x87    # The database storage is full
YSM_MEMORY_ERROR        = 0x88    # Memory read/write error
YSM_FUNCTION_DISABLED   = 0x89    # Funciton disabled via attribute(s)
YSM_KEY_STORAGE_LOCKED  = 0x8a    # Key storage locked
YSM_MISMATCH            = 0x8b    # Verification mismatch
YSM_INVALID_PARAMETER   = 0x8c    # Invalid parameter

def status2str(num):
    """ Return YubiHSM response status code as string. """
    known = {0x80: 'YSM_STATUS_OK',
             0x81: 'YSM_KEY_HANDLE_INVALID',
             0x82: 'YSM_AEAD_INVALID',
             0x83: 'YSM_OTP_INVALID',
             0x84: 'YSM_OTP_REPLAY',
             0x85: 'YSM_ID_DUPLICATE',
             0x86: 'YSM_ID_NOT_FOUND',
             0x87: 'YSM_DB_FULL',
             0x88: 'YSM_MEMORY_ERROR',
             0x89: 'YSM_FUNCTION_DISABLED',
             0x8a: 'YSM_KEY_STORAGE_LOCKED',
             0x8b: 'YSM_MISMATCH',
             0x8c: 'YSM_INVALID_PARAMETER',
             }

    if num in known:
        return known[num]
    return "0x%02x" % (num)

# HMAC flags
YSM_HMAC_SHA1_RESET		= 0x01     # Flag to indicate reset at first packet
YSM_HMAC_SHA1_FINAL		= 0x02     # Flag to indicate that the hash shall be calculated
YSM_HMAC_SHA1_TO_BUFFER	= 0x04     # Flag to transfer HMAC to buffer

# Commands
YSM_NULL			= 0x00
YSM_AEAD_GENERATE		= 0x01
YSM_BUFFER_AEAD_GENERATE	= 0x02
YSM_RANDOM_AEAD_GENERATE	= 0x03
YSM_AEAD_DECRYPT_CMP		= 0x04
YSM_DB_YUBIKEY_AEAD_STORE	= 0x05
YSM_AEAD_YUBIKEY_OTP_DECODE	= 0x06
YSM_DB_OTP_VALIDATE		= 0x07
YSM_DB_YUBIKEY_AEAD_STORE2	= 0x08
YSM_AES_ECB_BLOCK_ENCRYPT	= 0x0d
YSM_AES_ECB_BLOCK_DECRYPT	= 0x0e
YSM_AES_ECB_BLOCK_DECRYPT_CMP	= 0x0f
YSM_HMAC_SHA1_GENERATE		= 0x10
YSM_TEMP_KEY_LOAD		= 0x11
YSM_BUFFER_LOAD			= 0x20
YSM_BUFFER_RANDOM_LOAD		= 0x21
YSM_NONCE_GET			= 0x22
YSM_ECHO			= 0x23
YSM_RANDOM_GENERATE		= 0x24
YSM_RANDOM_RESEED		= 0x25
YSM_SYSTEM_INFO_QUERY		= 0x26
YSM_KEY_STORAGE_UNLOCK		= 0x27	# Deprecated in 1.0
YSM_HSM_UNLOCK			= 0x28
YSM_KEY_STORE_DECRYPT		= 0x29
YSM_MONITOR_EXIT		= 0x7f

def cmd2str(cmd):
    """ Return command as string. """
    known = {0x00: 'YSM_NULL',
             0x01: 'YSM_AEAD_GENERATE',
             0x02: 'YSM_BUFFER_AEAD_GENERATE',
             0x03: 'YSM_RANDOM_AEAD_GENERATE',
             0x04: 'YSM_AEAD_DECRYPT_CMP',
             0x05: 'YSM_DB_YUBIKEY_AEAD_STORE',
             0x06: 'YSM_AEAD_YUBIKEY_OTP_DECODE',
             0x07: 'YSM_DB_OTP_VALIDATE',
             0x08: 'YSM_DB_YUBIKEY_AEAD_STORE2',
             0x0d: 'YSM_AES_ECB_BLOCK_ENCRYPT',
             0x0e: 'YSM_AES_ECB_BLOCK_DECRYPT',
             0x0f: 'YSM_AES_ECB_BLOCK_DECRYPT_CMP',
             0x10: 'YSM_HMAC_SHA1_GENERATE',
             0x11: 'YSM_TEMP_KEY_LOAD',
             0x20: 'YSM_BUFFER_LOAD',
             0x21: 'YSM_BUFFER_RANDOM_LOAD',
             0x22: 'YSM_NONCE_GET',
             0x23: 'YSM_ECHO',
             0x24: 'YSM_RANDOM_GENERATE',
             0x25: 'YSM_RANDOM_RESEED',
             0x26: 'YSM_SYSTEM_INFO_QUERY',
             0x27: 'YSM_KEY_STORAGE_UNLOCK',
             0x28: 'YSM_HSM_UNLOCK',
             0x29: 'YSM_KEY_STORE_DECRYPT',
             0x7f: 'YSM_MONITOR_EXIT',
             }
    if cmd in known:
        return known[cmd]
    return "0x%02x" % (cmd)
